from datetime import timedelta
from pathlib import Path
from typing import Tuple

from airflow import DAG
from airflow.models.baseoperator import BaseOperator
from airflow.operators.empty import EmptyOperator

import opendbt


class OpenDbtExecutorOperator(BaseOperator):
    """
    An Airflow operator for executing dbt commands.
    """

    def __init__(self,
                 project_dir: Path,
                 command: str,
                 target: str = None,
                 profiles_dir: Path = None,
                 select: str = None,
                 args: list = None,
                 # without using subprocess airflow randomly gets deadlock
                 use_subprocess: bool = True,
                 execution_timeout=timedelta(minutes=60), **kwargs) -> None:
        super().__init__(execution_timeout=execution_timeout, **kwargs)

        self.project_dir: Path = project_dir
        self.command = command
        self.profiles_dir: Path = profiles_dir
        self.target = target
        self.use_subprocess = use_subprocess
        self.args = args if args else []

        if select:
            self.args += ["--select", select]

        # use separate colour for test and other executions
        if self.command == "test":
            self.ui_color = "#1CB1C2"
        else:
            self.ui_color = "#0084ff"

    def execute(self, context):
        """
        Execute the dbt command.
        """
        runner = opendbt.OpenDbtProject(project_dir=self.project_dir,
                                        profiles_dir=self.profiles_dir,
                                        target=self.target)
        runner.run(command=self.command, args=self.args, use_subprocess=self.use_subprocess)


# pylint: disable=too-many-locals, too-many-branches
class OpenDbtAirflowProject(opendbt.OpenDbtProject):

    def load_dbt_tasks(self,
                       dag: DAG,
                       start_node: BaseOperator = None,
                       end_node: BaseOperator = None,
                       tag: str = None,
                       resource_type="all",
                       include_dbt_seeds=False,
                       include_singular_tests=False) -> Tuple[BaseOperator, BaseOperator]:
        """
        This method is used to add dbt tasks to Given DAG.

        Parameters:
        dag (DAG): The Airflow DAG object where the dbt tasks will be added.
        start_node (BaseOperator, optional): The starting node of the DAG. If not provided, an EmptyOperator will be used.
        end_node (BaseOperator, optional): The ending node of the DAG. If not provided, an EmptyOperator will be used.
        tag (str, optional): The tag to filter the dbt tasks. If provided, only tasks with this tag will be added to the DAG.
        resource_type (str, optional): The type of dbt resource to run. It can be "all", "model", or "test". Default is "all".
        run_dbt_seeds (bool, optional): A flag to indicate whether to run dbt seeds before all other dbt jobs. Default is False.

        Returns:
        Tuple[BaseOperator, BaseOperator]: The start and end nodes of the DAG after adding the dbt tasks.
        """

        start_node = start_node if start_node else EmptyOperator(task_id='dbt-%s-start' % self.project_dir.name,
                                                                 dag=dag)
        end_node = end_node if end_node else EmptyOperator(task_id='dbt-%s-end' % self.project_dir.name, dag=dag)

        if include_dbt_seeds:
            # add dbt seeds job after start node abd before all other dbt jobs
            first_node = start_node
            start_node = OpenDbtExecutorOperator(dag=dag,
                                                 task_id="dbt-seeds",
                                                 project_dir=self.project_dir,
                                                 profiles_dir=self.profiles_dir,
                                                 target=self.target,
                                                 command="seed"
                                                 )
            start_node.set_upstream(first_node)

        manifest = self.manifest()
        dbt_tasks = {}
        # create all the jobs. granular as one job per model/table
        for key, node in manifest.nodes.items():
            if tag and tag not in node.tags:
                self.log.debug(
                    f"Skipping node:{node.name} because it dont have desired desired-tag={tag} node-tags={node.tags}")
                # LOG DEBUG OR TRACE here print(f" tag:{tag}  NOT in {node.tags} SKIPP {node.name}")
                continue  # skip if the node don't have the desired tag

            if resource_type == "test" and not str(node.name).startswith("source_"):
                if node.resource_type == "test":
                    dbt_tasks[node.unique_id] = OpenDbtExecutorOperator(dag=dag,
                                                                        task_id=node.unique_id,
                                                                        project_dir=self.project_dir,
                                                                        profiles_dir=self.profiles_dir,
                                                                        target=self.target,
                                                                        command="test",
                                                                        select=node.name
                                                                        )
                if node.resource_type == "model":
                    dbt_tasks[node.unique_id] = EmptyOperator(dag=dag, task_id=node.unique_id)

            if node.resource_type == "model" and resource_type in ["all", "model"]:
                # NOTE `build` command also runs the tests that's why are skipping tests for models below
                dbt_tasks[node.unique_id] = OpenDbtExecutorOperator(dag=dag,
                                                                    task_id=node.unique_id,
                                                                    project_dir=self.project_dir,
                                                                    profiles_dir=self.profiles_dir,
                                                                    target=self.target,
                                                                    command="build",
                                                                    select=node.name
                                                                    )

            if node.resource_type == "test" and str(node.name).startswith("source_") and resource_type in ["all",
                                                                                                           "test"]:
                # we are skipping model tests because they are included above with model execution( `build` command)
                # source table tests
                dbt_tasks[node.unique_id] = OpenDbtExecutorOperator(dag=dag,
                                                                    task_id=node.unique_id,
                                                                    project_dir=self.project_dir,
                                                                    profiles_dir=self.profiles_dir,
                                                                    target=self.target,
                                                                    command="test",
                                                                    select=node.name
                                                                    )

        # set upstream dependencies using dbt dependencies
        for key, node in manifest.nodes.items():
            if tag and tag not in node.tags:
                continue  # skip if the node don't have the desired tag
            if node.unique_id in dbt_tasks:  # node.resource_type == "model" or True or
                task = dbt_tasks[node.unique_id]
                if node.depends_on_nodes:
                    for upstream_id in node.depends_on_nodes:
                        if upstream_id in dbt_tasks:
                            self.log.debug(f"Setting upstream  of {task.task_id} -> {upstream_id}")
                            task.set_upstream(dbt_tasks[upstream_id])

        singular_tests = None
        if include_singular_tests:
            singular_tests = OpenDbtExecutorOperator(dag=dag,
                                                     task_id=f"{self.project_dir.name}_singular_tests",
                                                     project_dir=self.project_dir,
                                                     profiles_dir=self.profiles_dir,
                                                     target=self.target,
                                                     command="test",
                                                     select="test_type:singular"
                                                     )
        for k, task in dbt_tasks.items():
            if not task.downstream_task_ids:
                # set downstream dependencies for the end nodes.
                self.log.debug(f"Setting downstream  of {task.task_id} -> {end_node.task_id}")

                if include_singular_tests and singular_tests:
                    task.set_downstream(singular_tests)
                else:
                    task.set_downstream(end_node)

            if not task.upstream_task_ids:
                # set upstream dependencies for the nodes which don't have upstream dependency
                self.log.debug(f"Setting upstream  of {task.task_id} -> {start_node}")
                task.set_upstream(start_node)

        if include_singular_tests:
            singular_tests.set_downstream(end_node)
        return start_node, end_node



import dlt
from dlt.pipeline import TPipeline


@dlt.resource(
    columns={"event_tstamp": {"data_type": "timestamp", "precision": 3}},
    primary_key="event_id",
)
def events():
    yield [{"event_id": 1, "event_tstamp": "2024-07-30T10:00:00.123"},
           {"event_id": 2, "event_tstamp": "2025-02-30T10:00:00.321"}]


def model(dbt, pipeline: TPipeline):
    """

    :param dbt:
    :param pipeline: Pre-configured dlt pipeline. dlt target connection and dataset is pre-set using the model config!
    :return:
    """
    dbt.config(materialized="executedlt")
    print("========================================================")
    print(f"INFO: DLT Pipeline pipeline_name:{pipeline.pipeline_name}")
    print(f"INFO: DLT Pipeline dataset_name:{pipeline.dataset_name}")
    print(f"INFO: DLT Pipeline staging:{pipeline.staging}")
    print(f"INFO: DLT Pipeline destination:{pipeline.destination}")
    print(f"INFO: DLT Pipeline _pipeline_storage:{pipeline._pipeline_storage}")
    print(f"INFO: DLT Pipeline _schema_storage:{pipeline._schema_storage}")
    print(f"INFO: DLT Pipeline state:{pipeline.state}")
    print("========================================================")
    load_info = pipeline.run(events())
    print(load_info)
    row_counts = pipeline.last_trace.last_normalize_info
    print(row_counts)
    print("========================================================")
    return None



from pathlib import Path
from unittest import TestCase

from airflow import DAG
from airflow.operators.empty import EmptyOperator
from airflow.utils.dates import days_ago

from opendbt.airflow import OpenDbtAirflowProject


class TestOpenDbtProject(TestCase):
    RESOURCES_DIR = Path(__file__).parent.joinpath("resources")
    DBTTEST_DIR = RESOURCES_DIR.joinpath("dbttest")
    TEST_DAG = DAG(
        dag_id='dbt_test_workflow',
        schedule_interval=None,
        start_date=days_ago(3),
        catchup=False,
        max_active_runs=1
    )

    def test_run_dbt_as_airflow_task(self):
        with self.TEST_DAG as dag:
            # load dbt jobs to airflow dag
            start = EmptyOperator(task_id="start")
            end = EmptyOperator(task_id="end")
            p = OpenDbtAirflowProject(project_dir=self.DBTTEST_DIR, profiles_dir=self.DBTTEST_DIR, target='dev')
            p.load_dbt_tasks(dag=dag, start_node=start, end_node=end, include_singular_tests=True,
                             include_dbt_seeds=True)

            for j in dag.tasks:
                # don't run the model we created to fail
                if 'my_failing_dbt_model' in j.task_id:
                    continue
                # run all dbt tasks trough airflow operator
                j.execute({})



import subprocess


class Utils():

    @staticmethod
    def runcommand(command: list, shell=False):
        with subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, bufsize=1,
                              universal_newlines=True, shell=shell) as p:
            for line in p.stdout:
                if line:
                    print(line.strip())

        if p.returncode != 0:
            raise subprocess.CalledProcessError(p.returncode, p.args)



from pathlib import Path
from unittest import TestCase

from opendbt import OpenDbtProject


class TestOpenDbtProject(TestCase):
    RESOURCES_DIR = Path(__file__).parent.joinpath("resources")
    DBTTEST_DIR = RESOURCES_DIR.joinpath("dbttest")

    def test_run_compile(self):
        dp = OpenDbtProject(project_dir=self.DBTTEST_DIR, profiles_dir=self.DBTTEST_DIR)
        dp.run(command="compile")

    def test_run_run(self):
        dp = OpenDbtProject(project_dir=self.DBTTEST_DIR, profiles_dir=self.DBTTEST_DIR)
        dp.run(command="run", args=['--select', 'my_first_dbt_model+', "--exclude", "my_failing_dbt_model"],
               use_subprocess=True)

    def test_project_attributes(self):
        dp = OpenDbtProject(project_dir=self.DBTTEST_DIR, profiles_dir=self.DBTTEST_DIR)
        self.assertEqual(dp.project.project_name, "dbttest")
        self.assertEqual(dp.project.profile_name, "dbttest")
        self.assertEqual(dp.project_vars, {'dbt_custom_adapter': 'opendbt.examples.DuckDBAdapterV2Custom'})



import dbt
from packaging.version import Version


def patch_dbt():
    # ================================================================================================================
    # Monkey Patching! Override dbt lib code with new one
    # ================================================================================================================
    dbt_version = Version(dbt.version.get_installed_version().to_version_string(skip_matcher=True))
    if Version("1.6.0") <= dbt_version < Version("1.7.0"):
        from opendbt.dbt.v17.task.docs.generate import OpenDbtGenerateTask
        dbt.task.generate.GenerateTask = OpenDbtGenerateTask
        from opendbt.dbt.v17.adapters.factory import OpenDbtAdapterContainer
        dbt.adapters.factory.FACTORY = OpenDbtAdapterContainer()
        from opendbt.dbt.v17.task.run import ModelRunner
        dbt.task.run.ModelRunner = ModelRunner
    elif Version("1.7.0") <= dbt_version < Version("1.8.0"):
        from opendbt.dbt.v17.task.docs.generate import OpenDbtGenerateTask
        dbt.task.generate.GenerateTask = OpenDbtGenerateTask
        from opendbt.dbt.v17.adapters.factory import OpenDbtAdapterContainer
        dbt.adapters.factory.FACTORY = OpenDbtAdapterContainer()
        from opendbt.dbt.v17.task.run import ModelRunner
        dbt.task.run.ModelRunner = ModelRunner
    elif Version("1.8.0") <= dbt_version < Version("1.9.0"):
        from opendbt.dbt.v18.task.docs.generate import OpenDbtGenerateTask
        dbt.task.docs.generate.GenerateTask = OpenDbtGenerateTask
        from opendbt.dbt.v18.adapters.factory import OpenDbtAdapterContainer
        dbt.adapters.factory.FACTORY = OpenDbtAdapterContainer()
        from opendbt.dbt.v18.task.run import ModelRunner
        dbt.task.run.ModelRunner = ModelRunner
    elif Version("1.9.0") >= dbt_version < Version("1.10.0"):
        from opendbt.dbt.v18.task.docs.generate import OpenDbtGenerateTask
        dbt.task.docs.generate.GenerateTask = OpenDbtGenerateTask
        from opendbt.dbt.v18.adapters.factory import OpenDbtAdapterContainer
        dbt.adapters.factory.FACTORY = OpenDbtAdapterContainer()
        from opendbt.dbt.v18.task.run import ModelRunner
        dbt.task.run.ModelRunner = ModelRunner
    else:
        raise Exception(
            f"Unsupported dbt version {dbt_version}, please make sure dbt version is supported/integrated by opendbt")

    # shared code patches
    import opendbt.dbt.shared.cli.main
    dbt.cli.main.sqlfluff = opendbt.dbt.shared.cli.main.sqlfluff
    dbt.cli.main.sqlfluff_lint = opendbt.dbt.shared.cli.main.sqlfluff_lint
    dbt.cli.main.sqlfluff_fix = opendbt.dbt.shared.cli.main.sqlfluff_fix



import logging
import os
import sys
from pathlib import Path

from dbt.cli.main import dbtRunner as DbtCliRunner
from dbt.cli.main import dbtRunnerResult
from dbt.config import PartialProject
from dbt.contracts.graph.manifest import Manifest
from dbt.contracts.results import RunResult
from dbt.exceptions import DbtRuntimeError

from opendbt.dbt import patch_dbt
from opendbt.utils import Utils

######################
patch_dbt()


######################

class OpenDbtLogger:
    _log = None

    @property
    def log(self) -> logging.Logger:
        if self._log is None:
            self._log = logging.getLogger(name="opendbt")
            if not self._log.hasHandlers():
                handler = logging.StreamHandler(sys.stdout)
                formatter = logging.Formatter("[%(asctime)s] {%(filename)s:%(lineno)d} %(levelname)s - %(message)s")
                handler.setFormatter(formatter)
                handler.setLevel(logging.INFO)
                self._log.addHandler(handler)
        return self._log


class OpenDbtCli:

    @staticmethod
    def run(args: list, callbacks: list = None) -> dbtRunnerResult:
        """
        Run dbt with the given arguments.

        :param callbacks:
        :param args: The arguments to pass to dbt.
        :return: The result of the dbt run.
        """
        callbacks = callbacks if callbacks else []
        # https://docs.getdbt.com/reference/programmatic-invocations
        dbtcr = DbtCliRunner(callbacks=callbacks)
        result: dbtRunnerResult = dbtcr.invoke(args)
        if result.success:
            return result

        if result.exception:
            raise result.exception

        # take error message and raise it as exception
        for _result in result.result:
            _result: RunResult
            _result_messages = ""
            if _result.status == 'error':
                _result_messages += f"{_result_messages}\n"
            if _result_messages:
                raise DbtRuntimeError(msg=_result.message)

        raise DbtRuntimeError(msg=f"DBT execution failed!")


class OpenDbtProject(OpenDbtLogger):
    """
    This class is used to take action on a dbt project.
    """

    DEFAULT_TARGET = 'dev'  # development

    def __init__(self, project_dir: Path, target: str = None, profiles_dir: Path = None, args: list = None):
        super().__init__()
        self.project_dir: Path = project_dir
        self.profiles_dir: Path = profiles_dir
        self.target: str = target if target else self.DEFAULT_TARGET
        self.args = args if args else []
        self._project: PartialProject = None

    @property
    def project(self) -> PartialProject:
        if not self._project:
            self._project = PartialProject.from_project_root(project_root=self.project_dir.as_posix(),
                                                             verify_version=True)

        return self._project

    @property
    def project_dict(self) -> dict:
        return self.project.project_dict

    @property
    def project_vars(self) -> dict:
        """
        :return: dict: Variables defined in the `dbt_project.yml` file, `vars`.
                Note:
                    This method only retrieves global project variables specified within the `dbt_project.yml` file.
                    Variables passed via command-line arguments are not included in the returned dictionary.
        """
        if 'vars' in self.project_dict:
            return self.project_dict['vars']

        return {}

    def run(self, command: str = "build", target: str = None, args: list = None, use_subprocess: bool = False,
            write_json: bool = False) -> dbtRunnerResult:

        run_args = args if args else []
        run_args += ["--target", target if target else self.target]
        run_args += ["--project-dir", self.project_dir.as_posix()]
        if self.profiles_dir:
            run_args += ["--profiles-dir", self.profiles_dir.as_posix()]
        run_args = [command] + run_args + self.args
        if write_json:
            run_args.remove("--no-write-json")

        if use_subprocess:
            shell = False
            self.log.info(f"Working dir: {os.getcwd()}")
            py_executable = sys.executable if sys.executable else 'python'
            self.log.info(f"Python executable: {py_executable}")
            __command = [py_executable, '-m', 'opendbt'] + run_args
            self.log.info(f"Running command (shell={shell}) `{' '.join(__command)}`")
            Utils.runcommand(command=__command)
            return None

        self.log.info(f"Running `dbt {' '.join(run_args)}`")
        return OpenDbtCli.run(args=run_args)

    def manifest(self, partial_parse=True, no_write_manifest=True) -> Manifest:
        args = []
        if partial_parse:
            args += ["--partial-parse"]
        if no_write_manifest:
            args += ["--no-write-json"]

        result = self.run(command="parse", args=args)
        if isinstance(result.result, Manifest):
            return result.result

        raise Exception(f"DBT execution did not return Manifest object. returned:{type(result.result)}")

    def generate_docs(self, args: list = None):
        _args = ["generate"] + args if args else []
        self.run(command="docs", args=_args)



from pathlib import Path
from unittest import TestCase

from opendbt import OpenDbtProject


class TestOpenDbtProject(TestCase):
    RESOURCES_DIR = Path(__file__).parent.joinpath("resources")
    DBTTEST_DIR = RESOURCES_DIR.joinpath("dbttest")

    def test_run_executesql_materialization(self):
        dp = OpenDbtProject(project_dir=self.DBTTEST_DIR, profiles_dir=self.DBTTEST_DIR)
        dp.run(command="run", args=['--select', 'my_executesql_dbt_model'])



import importlib
import sys
import tempfile
from multiprocessing.context import SpawnContext
from typing import Dict

from dbt.adapters.base import available
from dbt.adapters.duckdb import DuckDBAdapter


class DuckDBAdapterV2Custom(DuckDBAdapter):

    @available
    def submit_local_python_job(self, parsed_model: Dict, compiled_code: str):
        connection = self.connections.get_if_exists()
        if not connection:
            connection = self.connections.get_thread_connection()

        with tempfile.NamedTemporaryFile(suffix=f'.py', delete=True) as model_file:
            model_file.write(compiled_code.lstrip().encode('utf-8'))
            model_file.flush()
            print(f"Created temp py file {model_file.name}")
            # load and execute python code!
            model_name = parsed_model['name']
            # Load the module spec
            spec = importlib.util.spec_from_file_location(model_name, model_file.name)
            # Create a module object
            module = importlib.util.module_from_spec(spec)
            # Load the module
            sys.modules[model_name] = module
            spec.loader.exec_module(module)
            dbt = module.dbtObj(None)
            # Access and call `model` function of the model!
            # IMPORTANT: here we are passing down duckdb session from the adapter to the model
            module.model(dbt=dbt, session=connection.handle)
            model_file.close()

    @available
    def submit_local_dlt_job(self, parsed_model: Dict, compiled_code: str):
        connection = self.connections.get_if_exists()
        if not connection:
            connection = self.connections.get_thread_connection()

        import dlt
        # IMPORTANT: here we are pre-configuring and preparing dlt.pipeline for the model!
        _pipeline = dlt.pipeline(
            pipeline_name=str(parsed_model['unique_id']).replace(".", "-"),
            destination=dlt.destinations.duckdb(connection.handle._env.conn),
            dataset_name=parsed_model['schema'],
            dev_mode=False,
        )

        with tempfile.NamedTemporaryFile(suffix=f'.py', delete=True) as model_file:
            model_file.write(compiled_code.lstrip().encode('utf-8'))
            model_file.flush()
            print(f"Created temp py file {model_file.name}")
            # load and execute python code!
            model_name = parsed_model['name']
            # Load the module spec
            spec = importlib.util.spec_from_file_location(model_name, model_file.name)
            # Create a module object
            module = importlib.util.module_from_spec(spec)
            # Load the module
            sys.modules[model_name] = module
            spec.loader.exec_module(module)
            dbt = module.dbtObj(None)
            # IMPORTANT: here we are passing down duckdb session from the adapter to the model
            module.model(dbt=dbt, pipeline=_pipeline)
            model_file.close()

# NOTE! used for testing
class DuckDBAdapterTestingOnlyDbt17(DuckDBAdapter):
    def __init__(self, config) -> None:
        print(f"WARNING: Using User Provided DBT Adapter: {type(self).__module__}.{type(self).__name__}")
        # pylint: disable=no-value-for-parameter
        super().__init__(config=config)
        raise Exception("Custom user defined test adapter activated, test exception")


# NOTE! used for testing
class DuckDBAdapterTestingOnlyDbt18(DuckDBAdapter):
    def __init__(self, config, mp_context: SpawnContext) -> None:
        print(f"WARNING: Using User Provided DBT Adapter: {type(self).__module__}.{type(self).__name__}")
        super().__init__(config=config, mp_context=mp_context)
        raise Exception("Custom user defined test adapter activated, test exception")



from pathlib import Path

from airflow import DAG
from airflow.operators.empty import EmptyOperator
from airflow.utils.dates import days_ago

from opendbt.airflow import OpenDbtAirflowProject

default_args = {
    'owner': 'airflow',
    'depends_on_past': False,
    'email_on_failure': False,
    'email_on_retry': False,
    'retries': 1
}

with DAG(
        dag_id='dbt_tests_workflow',
        default_args=default_args,
        description='DAG To run dbt tests',
        schedule_interval=None,
        start_date=days_ago(3),
        catchup=False,
        max_active_runs=1
) as dag:
    start = EmptyOperator(task_id="start")
    end = EmptyOperator(task_id="end")

    DBT_PROJ_DIR = Path("/opt/dbttest")

    p = OpenDbtAirflowProject(project_dir=DBT_PROJ_DIR, profiles_dir=DBT_PROJ_DIR, target='dev')
    p.load_dbt_tasks(dag=dag, start_node=start, end_node=end, resource_type='test')



from pathlib import Path
from unittest import TestCase

from opendbt import OpenDbtProject


class TestOpenDbtProject(TestCase):
    RESOURCES_DIR = Path(__file__).parent.joinpath("resources")
    DBTTEST_DIR = RESOURCES_DIR.joinpath("dbttest")

    def test_run_executepython_materialization(self):
        dp = OpenDbtProject(project_dir=self.DBTTEST_DIR, profiles_dir=self.DBTTEST_DIR,
                            args=['--vars', 'dbt_custom_adapter: opendbt.examples.DuckDBAdapterV2Custom'])
        dp.run(command="run", args=['--select', 'my_executepython_model'])

    def test_run_executepython_dlt_pipeline(self):
        dp = OpenDbtProject(project_dir=self.DBTTEST_DIR, profiles_dir=self.DBTTEST_DIR,
                            args=['--vars', 'dbt_custom_adapter: opendbt.examples.DuckDBAdapterV2Custom'])
        dp.run(command="run", args=['--select', 'my_executepython_dlt_model'])

    def test_run_executepython_materialization_subprocess(self):
        dp = OpenDbtProject(project_dir=self.DBTTEST_DIR, profiles_dir=self.DBTTEST_DIR,
                            args=['--vars', 'dbt_custom_adapter: opendbt.examples.DuckDBAdapterV2Custom'])
        dp.run(command="run", args=['--select', 'my_executepython_model'], use_subprocess=True)



import subprocess
import unittest
from pathlib import Path
from time import sleep

from testcontainers.compose import DockerCompose
from testcontainers.core.waiting_utils import wait_for_logs


@unittest.skip("Manual test")
class TestAirflowBase(unittest.TestCase):
    """
    Test class used to do airflow tests.
    uses airflow docker image and mounts current code into it.
    login is disabled all users can access the UI as Admin. Airflow is set up as Public
    """
    _compose: DockerCompose = None
    resources_dir = Path(__file__).parent.joinpath('resources')

    @classmethod
    def setUpClass(cls):
        cls._compose = DockerCompose(cls.resources_dir.joinpath('airflow').as_posix(),
                                     compose_file_name="docker-compose.yaml",
                                     build=True
                                     )
        cls._compose.stop()
        cls._compose.start()
        # cls._compose.wait_for(url="http://localhost:8080/health")
        wait_for_logs(cls._compose, 'Added Permission menu access on Configurations')
        wait_for_logs(cls._compose, 'Added user admin')

    @classmethod
    def tearDownClass(cls):
        print("Running tearDownClass")
        if cls._compose:
            cls._compose.stop()

    def __exit__(self, exc_type, exc_val, traceback):
        if self._compose:
            self._compose.stop()

    def _get_service_port(self, service, port):
        port_cmd = self._compose.docker_compose_command() + ["port", service, str(port)]
        output = subprocess.check_output(port_cmd, cwd=self._compose.filepath).decode("utf-8")
        result = str(output).rstrip().split(":")
        if len(result) != 2:
            raise Exception(f"Unexpected service info {output}. expecting `host:1234`")
        return result[-1]

    def test_start_airflow_local_and_wait(self):
        """
        used to deploy the code inside docker airflow locally. UI login is disabled and made public!
        useful to run local airflow with the new code changes and check the changes in airflow ui
        while its running all the code changes are reflected in airflow after short time.
        :return:
        """
        print(f"http://localhost:{self._get_service_port('airflow', 8080)}/home")
        print(f"http://localhost:{self._get_service_port('airflow', 8080)}/dbtdocsview")

        sleep(99999999)



import importlib
from importlib import import_module

from dbt.adapters import factory
from dbt.adapters.factory import Adapter
from dbt.events.functions import fire_event
from dbt.events.types import AdapterRegistered
from dbt.semver import VersionSpecifier


class OpenDbtAdapterContainer(factory.AdapterContainer):
    DBT_CUSTOM_ADAPTER_VAR = 'dbt_custom_adapter'
    def register_adapter(self, config: 'AdapterRequiredConfig') -> None:
        # ==== CUSTOM CODE ====
        # ==== END CUSTOM CODE ====
        adapter_name = config.credentials.type
        adapter_type = self.get_adapter_class_by_name(adapter_name)
        adapter_version = import_module(f".{adapter_name}.__version__", "dbt.adapters").version
        # ==== CUSTOM CODE ====
        custom_adapter_class_name: str = self.get_custom_adapter_config_value(config)
        if custom_adapter_class_name and custom_adapter_class_name.strip():
            # OVERRIDE DEFAULT ADAPTER BY USER GIVEN ADAPTER CLASS
            adapter_type = self.get_custom_adapter_class_by_name(custom_adapter_class_name)
        # ==== END CUSTOM CODE ====
        adapter_version_specifier = VersionSpecifier.from_version_string(
            adapter_version
        ).to_version_string()
        fire_event(
            AdapterRegistered(adapter_name=adapter_name, adapter_version=adapter_version_specifier)
        )
        with self.lock:
            if adapter_name in self.adapters:
                # this shouldn't really happen...
                return

            adapter: Adapter = adapter_type(config)  # type: ignore
            self.adapters[adapter_name] = adapter

    def get_custom_adapter_config_value(self, config: 'AdapterRequiredConfig') -> str:
        # FIRST: it's set as cli value: dbt run --vars {'dbt_custom_adapter': 'custom_adapters.DuckDBAdapterV1Custom'}
        if hasattr(config, 'cli_vars') and self.DBT_CUSTOM_ADAPTER_VAR in config.cli_vars:
            custom_adapter_class_name: str = config.cli_vars[self.DBT_CUSTOM_ADAPTER_VAR]
            if custom_adapter_class_name and custom_adapter_class_name.strip():
                return custom_adapter_class_name
        # SECOND: it's set inside dbt_project.yml
        if hasattr(config, 'vars') and self.DBT_CUSTOM_ADAPTER_VAR in config.vars.to_dict():
            custom_adapter_class_name: str = config.vars.to_dict()[self.DBT_CUSTOM_ADAPTER_VAR]
            if custom_adapter_class_name and custom_adapter_class_name.strip():
                return custom_adapter_class_name

        return None

    def get_custom_adapter_class_by_name(self, custom_adapter_class_name: str):
        if "." not in custom_adapter_class_name:
            raise ValueError(f"Unexpected adapter class name: `{custom_adapter_class_name}` ,"
                             f"Expecting something like:`my.sample.library.MyAdapterClass`")

        __module, __class = custom_adapter_class_name.rsplit('.', 1)
        try:
            user_adapter_module = importlib.import_module(__module)
            user_adapter_class = getattr(user_adapter_module, __class)
            return user_adapter_class
        except ModuleNotFoundError as mnfe:
            raise Exception(f"Module of provided adapter not found, provided: {custom_adapter_class_name}") from mnfe



import click
from dbt.cli import requires, params as p
from dbt.cli.main import global_flags, cli

from opendbt.dbt.shared.task.sqlfluff import SqlFluffTasks


# dbt docs
@cli.group()
@click.pass_context
@global_flags
def sqlfluff(ctx, **kwargs):
    """Generate or serve the documentation website for your project"""


# dbt docs generate
@sqlfluff.command("lint")
@click.pass_context
@global_flags
@p.defer
@p.deprecated_defer
@p.exclude
@p.favor_state
@p.deprecated_favor_state
@p.full_refresh
@p.indirect_selection
@p.profile
@p.profiles_dir
@p.project_dir
@p.resource_type
@p.select
@p.selector
@p.show
@p.state
@p.defer_state
@p.deprecated_state
@p.store_failures
@p.target
@p.target_path
@p.threads
@p.vars
@requires.postflight
@requires.preflight
@requires.profile
@requires.project
@requires.runtime_config
@requires.manifest(write=False)
def sqlfluff_lint(ctx, **kwargs):
    """Generate the documentation website for your project"""
    task = SqlFluffTasks(
        ctx.obj["flags"],
        ctx.obj["runtime_config"],
        ctx.obj["manifest"],
    )

    results = task.lint()
    success = task.interpret_results(results)
    return results, success


# dbt docs generate
@sqlfluff.command("fix")
@click.pass_context
@global_flags
@p.defer
@p.deprecated_defer
@p.exclude
@p.favor_state
@p.deprecated_favor_state
@p.full_refresh
@p.indirect_selection
@p.profile
@p.profiles_dir
@p.project_dir
@p.resource_type
@p.select
@p.selector
@p.show
@p.state
@p.defer_state
@p.deprecated_state
@p.store_failures
@p.target
@p.target_path
@p.threads
@p.vars
@requires.postflight
@requires.preflight
@requires.profile
@requires.project
@requires.runtime_config
@requires.manifest(write=False)
def sqlfluff_fix(ctx, **kwargs):
    """Generate the documentation website for your project"""
    task = SqlFluffTasks(
        ctx.obj["flags"],
        ctx.obj["runtime_config"],
        ctx.obj["manifest"],
    )

    results = task.fix()
    success = task.interpret_results(results)
    return results, success



import argparse

from opendbt import OpenDbtCli


def main():
    p = argparse.ArgumentParser()
    _, args = p.parse_known_args()
    OpenDbtCli.run(args=args)


if __name__ == "__main__":
    main()



import unittest
from pathlib import Path
from unittest import TestCase

from opendbt import OpenDbtProject


class TestDbtDocs(TestCase):
    RESOURCES_DIR = Path(__file__).parent.joinpath("resources")
    DBTTEST_DIR = RESOURCES_DIR.joinpath("dbttest")

    def test_run_docs_generate(self):
        dp = OpenDbtProject(project_dir=self.DBTTEST_DIR, profiles_dir=self.DBTTEST_DIR)
        dp.run(command="docs", args=['generate'])

    @unittest.skip("reason for skipping")
    def test_run_docs_serve(self):
        dp = OpenDbtProject(project_dir=self.DBTTEST_DIR, profiles_dir=self.DBTTEST_DIR)
        dp.run(command="docs", args=['serve'])



import os
from datetime import datetime
from pathlib import Path
from typing import Optional

from dbt.config import RuntimeConfig
from dbt.contracts.results import (
    CatalogResults,
    CatalogArtifact, RunExecutionResult,
)
from dbt.task.compile import CompileTask
from sqlfluff.cli import commands
from sqlfluff.core import Linter, FluffConfig
from sqlfluff.core.linter import LintingResult
from sqlfluff_templater_dbt import DbtTemplater


class SqlFluffTasks(CompileTask):

    def __init__(self, args, config, manifest):
        super().__init__(args, config, manifest)

        self.sqlfluff_config = FluffConfig.from_path(path=self.config.project_root)

        templater_obj = self.sqlfluff_config._configs["core"]["templater_obj"]
        if isinstance(templater_obj, DbtTemplater):
            templater_obj: DbtTemplater
            self.config: RuntimeConfig
            templater_obj.project_root = self.config.project_root
            templater_obj.working_dir = self.config.project_root
        self.linter = Linter(self.sqlfluff_config)

    def get_result(self, elapsed_time: float, violations: list, num_violations: int):
        run_result = RunExecutionResult(
            results=[],
            elapsed_time=elapsed_time,
            generated_at=datetime.now(),
            # args=dbt.utils.args_to_dict(self.args),
            args={},
        )
        result = CatalogArtifact.from_results(
            nodes={},
            sources={},
            generated_at=datetime.now(),
            errors=violations if violations else None,
            compile_results=run_result,
        )
        if num_violations > 0:
            setattr(result, 'exception', Exception(f"Linting {num_violations} errors found!"))
            result.exception = Exception(f"Linting {num_violations} errors found!")

        return result

    def lint(self) -> CatalogArtifact:
        os.chdir(self.config.project_root)
        lint_result: LintingResult = self.linter.lint_paths(paths=(self.config.project_root,))
        result = self.get_result(lint_result.total_time, lint_result.get_violations(), lint_result.num_violations())
        if lint_result.num_violations() > 0:
            print(f"Linting {lint_result.num_violations()} errors found!")
            for error in lint_result.as_records():
                filepath = Path(error['filepath'])
                violations: list = error['violations']
                if violations:
                    print(f"File: {filepath.relative_to(self.config.project_root)}")
                    for violation in violations:
                        print(f"    {violation}")
                        # print(f"Code:{violation['code']} Line:{violation['start_line_no']}, LinePos:{violation['start_line_pos']} {violation['description']}")
        return result

    def fix(self) -> CatalogArtifact:
        os.chdir(self.config.project_root)
        lnt, formatter = commands.get_linter_and_formatter(cfg=self.sqlfluff_config)
        lint_result: LintingResult = lnt.lint_paths(
            paths=(self.config.project_root,),
            fix=True,
            apply_fixes=True
        )
        result = self.get_result(lint_result.total_time, [], 0)
        return result

    @classmethod
    def interpret_results(self, results: Optional[CatalogResults]) -> bool:
        if results is None:
            return False
        if hasattr(results, "errors") and results.errors:
            return False
        return True



import sys
from pathlib import Path
from unittest import TestCase

from dbt.version import get_installed_version as get_dbt_version
from packaging.version import Version

from opendbt import OpenDbtProject

DBT_VERSION = get_dbt_version()
class TestOpenDbtProject(TestCase):
    RESOURCES_DIR = Path(__file__).parent.joinpath("resources")
    DBTTEST_DIR = RESOURCES_DIR.joinpath("dbttest")

    def test_run_with_custom_adapter(self):
        if Version(DBT_VERSION.to_version_string(skip_matcher=True)) > Version("1.8.0"):
            dbt_custom_adapter = 'opendbt.examples.DuckDBAdapterTestingOnlyDbt18'
        else:
            dbt_custom_adapter = 'opendbt.examples.DuckDBAdapterTestingOnlyDbt17'

        dp = OpenDbtProject(project_dir=self.DBTTEST_DIR, profiles_dir=self.DBTTEST_DIR,
                            args=['--vars', f"{{'dbt_custom_adapter': '{dbt_custom_adapter}'}}"])
        with self.assertRaises(Exception) as context:
            sys.tracebacklimit = 0
            dp.run(command="compile")
        self.assertTrue("Custom user defined test adapter activated" in str(context.exception))
        with self.assertRaises(Exception) as context:
            sys.tracebacklimit = 0
            dp.run(command="compile")
        self.assertTrue("Custom user defined test adapter activated" in str(context.exception))

    def test_run_with_custom_adapter_mmodule_not_found(self):
        dp = OpenDbtProject(project_dir=self.DBTTEST_DIR, profiles_dir=self.DBTTEST_DIR,
                            args=['--vars', '{dbt_custom_adapter: not.exits.module.MyDbtTestAdapterV1}']
                            )
        with self.assertRaises(Exception) as context:
            sys.tracebacklimit = 0
            dp.run(command="compile")
        self.assertTrue("Module of provided adapter not found" in str(context.exception))

    def test_run_with_custom_adapter_class_not_found(self):
        dp = OpenDbtProject(project_dir=self.DBTTEST_DIR, profiles_dir=self.DBTTEST_DIR,
                            args=['--vars', '{dbt_custom_adapter: test_custom_adapter.NotExistsAdapterClass}']
                            )
        with self.assertRaises(Exception) as context:
            sys.tracebacklimit = 0
            dp.run(command="compile")
        self.assertTrue("as no attribute 'NotExistsAdapterClass'" in str(context.exception))

    def test_run_with_custom_adapter_wrong_name(self):
        dp = OpenDbtProject(project_dir=self.DBTTEST_DIR, profiles_dir=self.DBTTEST_DIR,
                            args=['--vars', 'dbt_custom_adapter: test_custom_adapterMyDbtTestAdapterV1']
                            )
        with self.assertRaises(Exception) as context:
            sys.tracebacklimit = 0
            dp.run(command="compile")
        self.assertTrue("Unexpected adapter class name" in str(context.exception))



from dbt.contracts.results import NodeStatus
from dbt.events.base_types import EventLevel
from dbt.events.functions import fire_event
from dbt.events.types import (
    LogModelResult,
)
from dbt.task import run


class ModelRunner(run.ModelRunner):

    def print_result_adapter_response(self, result):
        if hasattr(result, 'adapter_response') and result.adapter_response:
            if result.status == NodeStatus.Error:
                status = result.status
                level = EventLevel.ERROR
            else:
                status = result.message
                level = EventLevel.INFO
            fire_event(
                LogModelResult(
                    description=str(result.adapter_response),
                    status=status,
                    index=self.node_index,
                    total=self.num_nodes,
                    execution_time=result.execution_time,
                    node_info=self.node.node_info,
                ),
                level=level,
            )

    def print_result_line(self, result):
        super().print_result_line(result)
        self.print_result_adapter_response(result=result)



"""Default configuration for the Airflow webserver."""
from __future__ import annotations

import os

from flask_appbuilder.const import AUTH_DB

# from airflow.www.fab_security.manager import AUTH_LDAP
# from airflow.www.fab_security.manager import AUTH_OAUTH
# from airflow.www.fab_security.manager import AUTH_OID
# from airflow.www.fab_security.manager import AUTH_REMOTE_USER


basedir = os.path.abspath(os.path.dirname(__file__))

# Flask-WTF flag for CSRF
WTF_CSRF_ENABLED = True
WTF_CSRF_TIME_LIMIT = None

# ----------------------------------------------------
# AUTHENTICATION CONFIG
# ----------------------------------------------------
# For details on how to set up each of the following authentication, see
# http://flask-appbuilder.readthedocs.io/en/latest/security.html# authentication-methods
# for details.

# The authentication type
# AUTH_OID : Is for OpenID
# AUTH_DB : Is for database
# AUTH_LDAP : Is for LDAP
# AUTH_REMOTE_USER : Is for using REMOTE_USER from web server
# AUTH_OAUTH : Is for OAuth
AUTH_TYPE = AUTH_DB

# Uncomment to setup Full admin role name
# AUTH_ROLE_ADMIN = 'Admin'

# Uncomment and set to desired role to enable access without authentication
AUTH_ROLE_PUBLIC = 'Admin'

# Will allow user self registration
# AUTH_USER_REGISTRATION = True

# The recaptcha it's automatically enabled for user self registration is active and the keys are necessary
# RECAPTCHA_PRIVATE_KEY = PRIVATE_KEY
# RECAPTCHA_PUBLIC_KEY = PUBLIC_KEY

# Config for Flask-Mail necessary for user self registration
# MAIL_SERVER = 'smtp.gmail.com'
# MAIL_USE_TLS = True
# MAIL_USERNAME = 'yourappemail@gmail.com'
# MAIL_PASSWORD = 'passwordformail'
# MAIL_DEFAULT_SENDER = 'sender@gmail.com'

# The default user self registration role
# AUTH_USER_REGISTRATION_ROLE = "Public"

# When using OAuth Auth, uncomment to setup provider(s) info
# Google OAuth example:
# OAUTH_PROVIDERS = [{
#   'name':'google',
#     'token_key':'access_token',
#     'icon':'fa-google',
#         'remote_app': {
#             'api_base_url':'https://www.googleapis.com/oauth2/v2/',
#             'client_kwargs':{
#                 'scope': 'email profile'
#             },
#             'access_token_url':'https://accounts.google.com/o/oauth2/token',
#             'authorize_url':'https://accounts.google.com/o/oauth2/auth',
#             'request_token_url': None,
#             'client_id': GOOGLE_KEY,
#             'client_secret': GOOGLE_SECRET_KEY,
#         }
# }]

# When using LDAP Auth, setup the ldap server
# AUTH_LDAP_SERVER = "ldap://ldapserver.new"

# When using OpenID Auth, uncomment to setup OpenID providers.
# example for OpenID authentication
# OPENID_PROVIDERS = [
#    { 'name': 'Yahoo', 'url': 'https://me.yahoo.com' },
#    { 'name': 'AOL', 'url': 'http://openid.aol.com/<username>' },
#    { 'name': 'Flickr', 'url': 'http://www.flickr.com/<username>' },
#    { 'name': 'MyOpenID', 'url': 'https://www.myopenid.com' }]

# ----------------------------------------------------
# Theme CONFIG
# ----------------------------------------------------
# Flask App Builder comes up with a number of predefined themes
# that you can use for Apache Airflow.
# http://flask-appbuilder.readthedocs.io/en/latest/customizing.html#changing-themes
# Please make sure to remove "navbar_color" configuration from airflow.cfg
# in order to fully utilize the theme. (or use that property in conjunction with theme)
# APP_THEME = "bootstrap-theme.css"  # default bootstrap
# APP_THEME = "amelia.css"
# APP_THEME = "cerulean.css"
# APP_THEME = "cosmo.css"
# APP_THEME = "cyborg.css"
# APP_THEME = "darkly.css"
# APP_THEME = "flatly.css"
# APP_THEME = "journal.css"
# APP_THEME = "lumen.css"
# APP_THEME = "paper.css"
# APP_THEME = "readable.css"
# APP_THEME = "sandstone.css"
# APP_THEME = "simplex.css"
# APP_THEME = "slate.css"
# APP_THEME = "solar.css"
# APP_THEME = "spacelab.css"
# APP_THEME = "superhero.css"
# APP_THEME = "united.css"
# APP_THEME = "yeti.css"



from pathlib import Path


# pylint: disable=inconsistent-return-statements
def init_plugins_dbtdocs_page(dbt_docs_dir: Path):
    from airflow.plugins_manager import AirflowPlugin
    from flask import Blueprint
    from flask_appbuilder import BaseView, expose
    from flask import abort
    from airflow.www.auth import has_access
    from airflow.security import permissions

    class DBTDocsView(BaseView):
        route_base = "/dbt"
        default_view = "dbt_docs_index"

        @expose("/dbt_docs_index.html")  # type: ignore[misc]
        @has_access([(permissions.ACTION_CAN_READ, permissions.RESOURCE_WEBSITE)])
        def dbt_docs_index(self):
            if not dbt_docs_dir.joinpath("index.html").is_file():
                abort(404)
            else:
                return dbt_docs_dir.joinpath("index.html").read_text()
            # return self.render_template("index.html", content="")

        @expose("/catalog.json")  # type: ignore[misc]
        @has_access([(permissions.ACTION_CAN_READ, permissions.RESOURCE_WEBSITE)])
        def catalog(self):
            if not dbt_docs_dir.joinpath("catalog.json").is_file():
                abort(404)
            else:
                data = dbt_docs_dir.joinpath("catalog.json").read_text()
                return data, 200, {"Content-Type": "application/json"}
            # return self.render_template("index.html", content="")

        @expose("/manifest.json")  # type: ignore[misc]
        @has_access([(permissions.ACTION_CAN_READ, permissions.RESOURCE_WEBSITE)])
        def manifest(self):
            if not dbt_docs_dir.joinpath("manifest.json").is_file():
                abort(404)
            else:
                data = dbt_docs_dir.joinpath("manifest.json").read_text()
                return data, 200, {"Content-Type": "application/json"}
            # return self.render_template("index.html", content="")

    # Creating a flask blueprint to integrate the templates and static folder
    bp = Blueprint(
        "DBT Plugin",
        __name__,
        template_folder=dbt_docs_dir.as_posix(),
        static_folder=dbt_docs_dir.as_posix(),
        # static_url_path='/dbtdocsview'
    )


    class AirflowDbtDocsPlugin(AirflowPlugin):
        name = "DBT Docs Plugin"
        flask_blueprints = [bp]
        appbuilder_views = [{"name": "DBT Docs", "category": "", "view": DBTDocsView()}]

    return AirflowDbtDocsPlugin



from pathlib import Path
from unittest import TestCase

from dbt.exceptions import DbtRuntimeError

from opendbt import OpenDbtProject


class TestOpenDbtCli(TestCase):
    RESOURCES_DIR = Path(__file__).parent.joinpath("resources")
    DBTTEST_DIR = RESOURCES_DIR.joinpath("dbttest")

    def test_run_failed(self):
        dp = OpenDbtProject(project_dir=self.DBTTEST_DIR, profiles_dir=self.DBTTEST_DIR)
        with self.assertRaises(DbtRuntimeError) as context:
            dp.run(command="run", args=['--select', '+my_failing_dbt_model'])

        self.assertIn('Referenced column "non_exists_column" not found in FROM clause', str(context.exception.msg))



import dlt


@dlt.resource(
    columns={"event_tstamp": {"data_type": "timestamp", "precision": 3}},
    primary_key="event_id",
)
def events():
    yield [{"event_id": 1, "event_tstamp": "2024-07-30T10:00:00.123"},
           {"event_id": 2, "event_tstamp": "2025-02-30T10:00:00.321"}]


def model(dbt, session):
    dbt.config(materialized="executepython")
    print("========================================================")
    print(f"INFO: DLT Version:{dlt.version.__version__}")
    print(f"INFO: DBT Duckdb Session:{type(session)}")
    print(f"INFO: DBT Duckdb Connection:{type(session._env.conn)}")
    print("========================================================")
    p = dlt.pipeline(
        pipeline_name="dbt_dlt",
        destination=dlt.destinations.duckdb(session._env.conn),
        dataset_name=dbt.this.schema,
        dev_mode=False,
    )
    load_info = p.run(events())
    print(load_info)
    row_counts = p.last_trace.last_normalize_info
    print(row_counts)
    print("========================================================")
    return None



import shutil
from pathlib import Path

import click
from dbt.task.generate import GenerateTask


class OpenDbtGenerateTask(GenerateTask):

    def deploy_user_index_html(self):
        # run custom code
        target = Path(self.config.project_target_path).joinpath("index.html")
        for dir in self.config.docs_paths:
            index_html = Path(self.config.project_root).joinpath(dir).joinpath("index.html")
            if index_html.is_file() and index_html.exists():
                # override default dbt provided index.html with user index.html file
                shutil.copyfile(index_html, target)
                click.echo(f"Using user provided documentation page: {index_html.as_posix()}")
                break

    def run(self):
        # Call the original dbt run method
        super().run()
        self.deploy_user_index_html()



from pathlib import Path

from opendbt.airflow import plugin

# create public page on airflow server to serve DBT docs
airflow_dbtdocs_page = plugin.init_plugins_dbtdocs_page(Path("/opt/dbttest/target"))



from pathlib import Path
from unittest import TestCase

from opendbt import OpenDbtProject


class TestDbtSqlFluff(TestCase):
    RESOURCES_DIR = Path(__file__).parent.joinpath("resources")
    DBTTEST_DIR = RESOURCES_DIR.joinpath("dbttest")

    def test_run_sqlfluff_lint(self):
        dp = OpenDbtProject(project_dir=self.DBTTEST_DIR, profiles_dir=self.DBTTEST_DIR)
        dp.run(command="sqlfluff", args=['fix'])
        dp.run(command="sqlfluff", args=['lint'])

    def test_run_sqlfluff_fix(self):
        dp = OpenDbtProject(project_dir=self.DBTTEST_DIR, profiles_dir=self.DBTTEST_DIR)
        dp.run(command="sqlfluff", args=['fix'])
